package net.openseed.osaf.docgen.info;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;
import javax.persistence.Version;

import net.openseed.osaf.util.StringUtils;

import org.hibernate.dialect.Dialect;
import org.hibernate.dialect.MySQL5Dialect;

public class EntityInfo {

	Class entityClass;

	String entityName;
	String fullPackageClassName;
	String tableName;
	String sequenceName;
	List<String> indexes;
	String version;

	PKInfo pkInfo;
	List<AssociationInfo> ainfos;
	List<FieldInfo> finfos;
	
	Dialect dialect;

	public EntityInfo(Class entityClass, Dialect dialect) {
		if(dialect != null)
			this.dialect = dialect;
		else
			this.dialect = new MySQL5Dialect();
		this.entityClass = entityClass;
		indexes = new ArrayList<String>();
		ainfos = new ArrayList<AssociationInfo>();
		finfos = new ArrayList<FieldInfo>();

		entityName = entityClass.getSimpleName();
		fullPackageClassName = entityClass.getName();
		tableName = fillTableName();
		sequenceName = fillSeqName();
		indexes = fillIndexs();
		version = fillVersion();

		fillPKInfos();
		fillAssociationInfos();
		fillFiledInfos();
		
		fillDefaultValues();
	}

	private void fillDefaultValues() {
		if(StringUtils.isEmpty(tableName))
			tableName = entityName;
	}

	public EntityInfo(Class entityClass) {
		this(entityClass, null);
	}

	private void fillPKInfos() {
		Field idField = findFiledByAnnotation(entityClass, Id.class);
		pkInfo = new PKInfo(idField);
	}

	private void fillFiledInfos() {
		Field[] fs = entityClass.getDeclaredFields();
		for (Field f : fs) {
			if (f.getAnnotation(Column.class) != null) {
				finfos.add(new FieldInfo(f, dialect));
			}
		}

	}

	private void fillAssociationInfos() {
		Field[] fs = entityClass.getDeclaredFields();
		List<Class> as = Arrays.asList(new Class[] { OneToMany.class,
				OneToOne.class, ManyToMany.class, ManyToOne.class });

		for (Field f : fs) {
			for (Class a : as) {
				if (f.getAnnotation(a) != null) {
					ainfos.add(new AssociationInfo(f, a));
				}
			}
		}
	}

	private String fillVersion() {
		Field versionField = findFiledByAnnotation(entityClass, Version.class);
		if (versionField != null)
			return versionField.getName();
		return "";
	}

	private Field findFiledByAnnotation(Class entityClass, Class annotationClass) {
		Field[] fs = entityClass.getDeclaredFields();
		Field resultField = null;
		for (Field f : fs) {
			if (f.getAnnotation(annotationClass) != null)
				resultField = f;
		}
		return resultField;
	}

	private List<String> fillIndexs() {
		Table table = (Table) entityClass.getAnnotation(Table.class);
		UniqueConstraint[] ucs = null;

		if (table != null)
			ucs = table.uniqueConstraints();

		if (ucs != null) {
			return Arrays.asList(ucs[0].columnNames());
		}

		return null;
	}

	private String fillSeqName() {
		SequenceGenerator sg = (SequenceGenerator) entityClass
				.getAnnotation(SequenceGenerator.class);
		if (sg != null)
			return sg.name();
		return "";
	}

	private String fillTableName() {
		Table table = (Table) entityClass.getAnnotation(Table.class);
		if (table != null)
			return table.name();

		Entity entity = (Entity) entityClass.getAnnotation(Entity.class);
		if (entity != null) {
			return entity.name();
		}

		return "";
	}

	public Class getEntityClass() {
		return entityClass;
	}

	public String getEntityName() {
		return entityName;
	}

	public String getFullPackageClassName() {
		return fullPackageClassName;
	}

	public String getTableName() {
		return tableName;
	}

	public String getSequenceName() {
		return sequenceName;
	}

	public List<String> getIndexes() {
		return indexes;
	}

	public String getVersion() {
		return version;
	}

	public List<AssociationInfo> getAinfos() {
		return ainfos;
	}

	public List<FieldInfo> getFinfos() {
		return finfos;
	}

	public String getIndexesBySpace() {
		if (indexes != null && indexes.size() > 0) {
			StringBuilder builder = new StringBuilder();
			for (String index : indexes) {
				builder.append(index);
				builder.append(" ");
			}
			return builder.toString();
		}
		return "";
	}

	public PKInfo getPkInfo() {
		return pkInfo;
	}

}
